/*------------------------------------------------------------------------- 
 *            Example: Teensy sketch for AudioGrabberComplex256
 *   The sketch creates a a quadrature (complex) sine function at 500 Hz 
 *   using the Audio Library then "grabs" it within the loop() function for 
 *   user processing.  In this case the two components (real and imaginary) 
 *   of the waveform are simply displayed using the Serial Plotter 
 *   (Tools->SerialPlotter).  
 *   
 *   Note:  There is no garantee that you will grab contiguous length 256
 *          blocks of data fom the audio stream. The output buffer is 
 *          updated every 5.8 msec, so if your loop() processing time is 
 *          greater than that, you are in danger of skipping a block.
 *----------------------------------------------------------------------- */
#include <Audio.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <SerialFlash.h>
#include "AudioGrabberComplex256.h"
//
// -- Establish a length 512 data array to receive 256 "grabbed" 
//    complex data points in arm_math format [real, imag, real, imag, ...],
//    giving a total of 512 elements
int16_t  inBuff[512];
//
AudioControlSGTL5000     codec;
AudioSynthWaveformSine   sineReal;         
AudioSynthWaveformSine   sineImag;         
AudioGrabberComplex256   myDataGrabber;
// Teensy needs to declare an output even if is never used
AudioOutputI2S           output;
// Audio connections:
// Note: While this is a very simple example, the grabber can be placed
//       anywhere along a processing chain.
AudioConnection c1(sineReal,0, myDataGrabber,0);
AudioConnection c2(sineImag,0, myDataGrabber,1);
//
//--
void setup() {
  Serial.begin(115200);
  delay(2000);
  codec.enable();
  AudioMemory(12);
  codec.volume(0.5);
  //
  AudioNoInterrupts();
  sineReal.amplitude(0.5);
  sineReal.frequency(500);
  sineReal.phase(90.0);            // Cosine term (real)
  sineImag.amplitude(0.5);
  sineImag.frequency(500);
  sineImag.phase(0.0);             // Sine term   (imaginary)
  AudioInterrupts();
  //
}
// ---
void loop() {
  if(myDataGrabber.newDataAvailable()) {
    myDataGrabber.grab(inBuff);
    for(int i = 0; i< 256; i+=1){
      // You can now do what you wish with the data.  In this case we just
      // plot it using the Serial plotter.   Just make sure your Teensy and 
      // computer are fast enough to complete the plotting within the 5.8
      // msec update period
      Serial.print(inBuff[2*i]);       // Real part
      Serial.print("\t");
      Serial.println(inBuff[2*i+1]);   // Imaginary part
    }
  }
}
