/*------------------------------------------------------------------------- 
 *            Example: Teensy sketch for AudioGrabberReal256
 *   The sketch creates a sine function at 100 Hz using the Audio Library
 *   then "grabs" it within the loop() function for user processing.
 *   In this case the waveform is simply displayed using the Serial Plotter
 *   (Tools->SerialPlotter).  
 *   
 *   Note:  There is no garantee that you will grab contiguous length 256
 *          blocks of data fom the audio stream. The output buffer is 
 *          updated every 5.8 msec, so if your loop() processing time is 
 *          greater than that, you are in danger of skipping a block.
 *----------------------------------------------------------------------- */
#include <Audio.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <SerialFlash.h>
#include "AudioGrabberReal256.h"
//
// --- Establish a length 512 data array to receive 256 "grabbed" 
//     real data points.
int16_t  myBuff[256];
//
AudioControlSGTL5000     codec;
AudioSynthWaveformSine   sine;         
AudioGrabberReal256      myDataGrabber;
// -- Teensy Audio Library needs to declare an output even if is never used!!
AudioOutputI2S           output;
// -- Audio connections:
//    Note: While this is a very simple example, the grabber can be placed
//          anywhere along an Audio processing chain.
//          The "real" grabber has only one input.
AudioConnection c1(sine,0, myDataGrabber,0);
//
//--
void setup() {
  Serial.begin(115200);
  delay(2000);
  codec.enable();
  AudioMemory(12);
  codec.volume(0.5);
  //
  AudioNoInterrupts();
  sine.amplitude(0.5);
  sine.frequency(100);
  AudioInterrupts();
}
//--
void loop() {
  if(myDataGrabber.newDataAvailable()) {
    myDataGrabber.grab(myBuff);
    // --- You can now do what you wish with the data.  In this case we just
    //     plot it using the Serial plotter.   Just make sure your Teensy and 
    //     computer are fast enough to complete the plotting within the 5.8
    //     msec update period
    for(int i=0; i<256; i++){
      Serial.println(myBuff[i]);       // Display on the Serial plotter
    }
  }
}
